from gdriveplayer import GDrivePlayer
from exceptions import SearchLimitError, IDNotFoundError
from typing import Union, Optional, List
from utils import  plus_encode, jsonify, gen_ep_from_player_url

class Drama:

    def __init__(
                    self, 
                    id, 
                    title,
                    poster,
                    genre,
                    summary,
                    status,
                    type,
                    total_episode,
                    sub,
                    player_url
                ) -> None:

        self.id = id
        self.title = title
        self.poster = poster 
        self.genre = genre
        self.summary = summary
        self.status = status
        self.type = type
        self.total_episode = total_episode
        self.sub = sub
        self.player_url = player_url 


class LatestDrama:

    def __init__(
                    self, 
                    id, 
                    title,
                    poster,
                    genre,
                    status,
                    type,
                    total_episode,
                    sub,
                    player_url
                ) -> None:

        self.id = id
        self.title = title
        self.poster = poster 
        self.genre = genre
        self.status = status
        self.type = type
        self.total_episode = total_episode
        self.sub = sub
        self.player_url = player_url 


class GDrama(GDrivePlayer):

    def __init__(self) -> None:
        self.__url_Drama = super().url_drama

        super().__init__()

    def search(self, title: Optional[str] = '', limit: Optional[Union[int, str]] = 10, page: Optional[Union[int, str]] = 1) -> List[Drama]:
        if limit > 100:
            raise SearchLimitError(limit=limit)
        url = f"{self.__url_Drama}search?title={plus_encode(title)}&limit={limit}&page={page}"
        res = jsonify(super().request(url))
        DramaList = []
        for a in res:
            a["player_url"] = gen_ep_from_player_url(a["player_url"], a["total_episode"])
            DramaList.append(Drama(**a))

        return DramaList


    def latestDramas(self, limit: Optional[Union[str, int]] = 10, page: Optional[Union[str, int]] = 1, order: Optional[str] = "last_updated", sort: Optional[str] = "DESC") -> List[Drama]:
        if limit > 100:
            raise SearchLimitError(limit=limit)
        url = f"{self.__url_Drama}newest?limit={limit}&page={page}&order={order}&sort={sort}"
        res = jsonify(super().request(url))
        DramaList = []
        for a in res:
            DramaList.append(LatestDrama(**a))

        return DramaList


    def DramaDetail(self, id: Union[str, int]) -> Drama:
        url = f"{self.__url_Drama}id/{id}"
        try:
            res = jsonify(super().request(url))
            return Drama(**res[0])
        except KeyError:
            raise IDNotFoundError(id=id)


from gdriveplayer import GDrivePlayer
from exceptions import SearchLimitError, IDNotFoundError
from typing import Union, Optional, List
from utils import  plus_encode, jsonify

class Movie:

    def __init__(
                    self, 
                    title, 
                    year,
                    imdb,
                    poster,
                    genre,
                    runtime,
                    director,
                    country,
                    rating,
                    votes,
                    sub,
                    quality
                ) -> None:

        self.title = title
        self.year = year
        self.imdb = imdb 
        self.poster = poster
        self.genre = genre
        self.runtime = runtime
        self.director = director
        self.country = country
        self.rating = rating
        self.votes = votes
        self.sub = sub
        self.quality = quality


    def getPlayer(self) -> str:
        return f"http://database.gdriveplayer.us/player.php?imdb={self.imdb}"


class MovieIMDB:

    def __init__(
                    self, 
                    Title, 
                    Year,
                    Rated,
                    Released,
                    Runtime,
                    Genre,
                    Director,
                    Writer,
                    Actors,
                    Plot,
                    Language,
                    Country,
                    Awards,
                    Poster,
                    imdbRating,
                    imdbVotes,
                    imdbID,
                    Type,
                    Production,
                    Response,
                    player_url
                ) -> None:

        self.title = Title
        self.year = Year
        self.rated = Rated
        self.released = Released
        self.runtime = Runtime
        self.genre = Genre
        self.director = Director
        self.writer = Writer
        self.actors = Actors
        self.plot = Plot
        self.language = Language
        self.country = Country
        self.awards = Awards
        self.poster = Poster
        self.imdb_rating = imdbRating
        self.imdb_votes = imdbVotes
        self.imdb_id = imdbID
        self.type = Type
        self.production = Production
        self.response = Response
        self.player_url = player_url




class GMovie(GDrivePlayer):

    def __init__(self) -> None:
        self.__url_movie = super().url_movie

        super().__init__()

    def search(self, title: Optional[str] = '', year: Optional[Union[str, int]] = '', genre: Optional[str] = '', country: Optional[str] = '', actor: Optional[str] = '', limit: Optional[Union[int, str]] = 10, page: Optional[Union[int, str]] = 1) -> List[Movie]:
        if limit > 100:
            raise SearchLimitError(limit=limit)
        url = f"{self.__url_movie}search?title={plus_encode(title)}&limit={limit}&page={page}&year={year}&genre={genre}&country={country}&actor={actor}"
        res = jsonify(super().request(url))
        movieList = []
        for a in res:
            movieList.append(Movie(**a))

        return movieList


    def latestMovies(self, limit: Optional[Union[str, int]] = 10, page: Optional[Union[str, int]] = 1, order: Optional[str] = "last_updated", sort: Optional[str] = "DESC") -> List[Movie]:
        if limit > 100:
            raise SearchLimitError(limit=limit)
        url = f"{self.__url_movie}newest?limit={limit}&page={page}&order={order}&sort={sort}"
        res = jsonify(super().request(url))
        movieList = []
        for a in res:
            movieList.append(Movie(**a))

        return movieList


    def movieDetail(self, imdb_id: Union[str, int]) -> Movie:
        url = f"{self.__url_movie[:-6]}imdb/{imdb_id}"
        try:
            res = jsonify(super().request(url))
            return MovieIMDB(**res)
        except KeyError:
            raise IDNotFoundError(id=id)



import requests
from exceptions import StatusCodeError


class GDrivePlayer:

    url_anime =  "https://api.gdriveplayer.us/v1/animes/"
    url_movie = "https://api.gdriveplayer.us/v1/movie/"
    url_drama = "https://api.gdriveplayer.us/v1/drama/"
    url_series = "https://api.gdriveplayer.us/v2/series/"

    def request(self, url : str):
        try:
            res = requests.get(url)
            if res.status_code != 200:
                raise StatusCodeError
            else:
                return res.text
        except Exception as e:
            print(e)
            exit
        





class StatusCodeError(Exception):
    pass


class SearchLimitError(Exception):
    
    def __init__(self, limit) -> None:
        self.limit = limit
        self.message = f"Limit cannot exceed 100. But you requested {limit} Objects."
        super().__init__(self.message)


class IDNotFoundError(Exception):

    def __init__(self, id) -> None:
        self.message = f"The Specified ID {id} is not Found."
        super().__init__(self.message)



import json
from typing import Union, List

def plus_encode(query: str):
    return query.replace(' ', '+')


def jsonify(content: str):
    return json.loads(content)

    
def gen_ep_from_player_url(url :str, total_ep: Union[int, str]) -> List[str]:
    url = url[:url.index('{')]
    player_urls = []
    for i in range(1, int(total_ep)+1):
        player_urls.append(f"{url}{i}")
    return player_urls
    



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['GDrivePlayer']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'gdriveplayer',
    'version': '0.1.0',
    'description': 'A python wrapper for gdriveplayer.co API',
    'long_description': '# GDrivePlayerAPI\nA python wrapper for gdriveplayer.co API\n',
    'author': 'adenosinetp10',
    'author_email': 'adenosinetp10@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
